<?php

namespace Drupal\simple_sitemap\Form;

use Drupal\Core\Config\ConfigFactoryInterface;
use Drupal\Core\Datetime\DateFormatter;
use Drupal\simple_sitemap\Entity\SimpleSitemap;
use Drupal\simple_sitemap\Entity\SimpleSitemapType;
use Drupal\simple_sitemap\Queue\QueueWorker;
use Drupal\simple_sitemap\Settings;
use Symfony\Component\DependencyInjection\ContainerInterface;
use Drupal\Core\Form\FormStateInterface;
use Drupal\simple_sitemap\Manager\Generator as SimplesitemapOld;
use Drupal\Core\Database\Connection;

/**
 * Class SitemapsForm
 */
class SitemapsForm extends SimpleSitemapFormBase {

  /**
   * @var \Drupal\Core\Database\Connection
   */
  protected $db;

  /**
   * @var \Drupal\Core\Datetime\DateFormatter
   */
  protected $dateFormatter;

  protected $queueWorker;

  /**
   * SitemapsForm constructor.
   *
   * @param \Drupal\Core\Config\ConfigFactoryInterface $config_factory
   * @param SimplesitemapOld $generator
   * @param \Drupal\simple_sitemap\Settings $settings
   * @param \Drupal\simple_sitemap\Form\FormHelper $form_helper
   * @param \Drupal\Core\Database\Connection $database
   * @param \Drupal\Core\Datetime\DateFormatter $date_formatter
   * @param \Drupal\simple_sitemap\Queue\QueueWorker $queue_worker
   */
  public function __construct(
    ConfigFactoryInterface $config_factory,
    SimplesitemapOld $generator,
    Settings $settings,
    FormHelper $form_helper,
    Connection $database,
    DateFormatter $date_formatter,
    QueueWorker $queue_worker
  ) {
    parent::__construct(
      $config_factory,
      $generator,
      $settings,
      $form_helper
    );
    $this->db = $database;
    $this->dateFormatter = $date_formatter;
    $this->queueWorker = $queue_worker;
  }

  /**
   * {@inheritdoc}
   */
  public static function create(ContainerInterface $container) {
    return new static(
      $container->get('config.factory'),
      $container->get('simple_sitemap.generator'),
      $container->get('simple_sitemap.settings'),
      $container->get('simple_sitemap.form_helper'),
      $container->get('database'),
      $container->get('date.formatter'),
      $container->get('simple_sitemap.queue_worker')
    );
  }

  /**
   * {@inheritdoc}
   */
  public function getFormId(): string {
    return 'simple_sitemap_sitemaps_form';
  }

  /**
   * {@inheritdoc}
   */
  public function buildForm(array $form, FormStateInterface $form_state): array {

    $form['simple_sitemap_settings']['#prefix'] = FormHelper::getDonationText();
    $form['simple_sitemap_settings']['#attached']['library'][] = 'simple_sitemap/sitemaps';

    $form['simple_sitemap_settings']['status'] = [
      '#type' => 'fieldset',
      '#title' => $this->t('Sitemap status'),
      '#markup' => '<div class="description">' . $this->t('Sitemaps can be regenerated on demand here.') . '</div>',
      '#description' => $this->t('Variants can be configured <a href="@url">here</a>.', ['@url' => $GLOBALS['base_url'] . '/admin/config/search/simplesitemap/variants']),
    ];

    $form['simple_sitemap_settings']['status']['actions'] = [
      '#prefix' => '<div class="clearfix"><div class="form-item">',
      '#suffix' => '</div></div>',
    ];

    $form['simple_sitemap_settings']['status']['actions']['rebuild_queue_submit'] = [
      '#type' => 'submit',
      '#value' => $this->t('Rebuild queue'),
      '#submit' => ['::rebuildQueue'],
      '#validate' => [],
    ];

    $form['simple_sitemap_settings']['status']['actions']['regenerate_submit'] = [
      '#type' => 'submit',
      '#value' => $this->queueWorker->generationInProgress()
        ? $this->t('Resume generation')
        : $this->t('Rebuild queue & generate'),
      '#submit' => ['::generateSitemap'],
      '#validate' => [],
    ];

    $form['simple_sitemap_settings']['status']['progress'] = [
      '#prefix' => '<div class="clearfix">',
      '#suffix' => '</div>',
    ];

    $form['simple_sitemap_settings']['status']['progress']['title']['#markup'] = $this->t('Progress of sitemap regeneration');

    $total_count = $this->queueWorker->getInitialElementCount();
    if (!empty($total_count)) {
      $indexed_count = $this->queueWorker->getProcessedElementCount();
      $percent = round(100 * $indexed_count / $total_count);

      // With all results processed, there still may be some stashed results to be indexed.
      $percent = $percent === 100 && $this->queueWorker->generationInProgress() ? 99 : $percent;

      $index_progress = [
        '#theme' => 'progress_bar',
        '#percent' => $percent,
        '#message' => $this->t('@indexed out of @total queue items have been processed.<br>Each sitemap variant is published after all of its items have been processed.', ['@indexed' => $indexed_count, '@total' => $total_count]),
      ];
      $form['simple_sitemap_settings']['status']['progress']['bar']['#markup'] = render($index_progress);
    }
    else {
      $form['simple_sitemap_settings']['status']['progress']['bar']['#markup'] = '<div class="description">' . $this->t('There are no items to be indexed.') . '</div>';
    }

    foreach (SimpleSitemapType::loadMultiple() as $type_id => $sitemap_type) {
      if ($variants = \Drupal::entityTypeManager()->getStorage('simple_sitemap')->loadByProperties(['type' => $type_id])) {

        $form['simple_sitemap_settings']['status']['types'][$type_id] = [
          '#type' => 'details',
          '#title' => '<em>' . $sitemap_type->label() . '</em> ' . $this->t('sitemaps'),
          '#open' => count($variants) <= 5,
          '#description' => !empty($description = $sitemap_type->getDescription()) ? '<div class="description">' . $description . '</div>' : '',
        ];
        $form['simple_sitemap_settings']['status']['types'][$type_id]['table'] = [
          '#type' => 'table',
          '#header' => [$this->t('Variant'), $this->t('Status'), $this->t('Link count')],
          '#attributes' => ['class' => ['form-item', 'clearfix']],
        ];
        foreach ($variants as $variant) {
          /** @var \Drupal\simple_sitemap\Entity\SimpleSitemapInterface $variant */
          if (empty($variant->fromPublishedAndUnpublished()->getChunkCount())) {
            $row['name']['data']['#markup'] = '<span title="' . $variant->id() . '">' . $this->t($variant->label()) . '</span>';
            $row['status'] = $this->t('pending');
            $row['count'] = '';
          }
          else {
            switch ($variant->contentStatus()) {

              case SimpleSitemap::SITEMAP_UNPUBLISHED:
                $row['name']['data']['#markup'] = '<span title="' . $variant->id() . '">' . $this->t($variant->label()) . '</span>';
                $row['status'] = $this->t('generating');
                $row['count'] = '';
                break;

              case SimpleSitemap::SITEMAP_PUBLISHED:
              case SimpleSitemap::SITEMAP_PUBLISHED_GENERATING:
                $row['name']['data']['#markup'] = $this->t('<a href="@url" target="_blank">@variant</a>',
                  ['@url' => $variant->toUrl()->toString(), '@variant' => $this->t($variant->label())]
                );
                $row['status'] = $this->t(($variant->contentStatus() === SimpleSitemap::SITEMAP_PUBLISHED
                  ? 'published on @time'
                  : 'published on @time, regenerating'
                ), ['@time' => $this->dateFormatter->format($variant->fromPublished()->getCreated())]);
                // Once the sitemap has been regenerated after
                // simple_sitemap_update_8305() there will always be a link
                // count.
                $row['count'] = $variant->fromPublished()->getLinkCount() > 0
                  ? $variant->fromPublished()->getLinkCount()
                  : $this->t('unavailable');
                break;
            }
          }
          $form['simple_sitemap_settings']['status']['types'][$type_id]['table']['#rows'][$variant->id()] = isset($row) ? $row : [];
        }
      }
    }
    if (empty($form['simple_sitemap_settings']['status']['types'])) {
      $form['simple_sitemap_settings']['status']['types']['#markup'] = $this->t('No variants have been defined');
    }

    return $form;
  }

  /**
   * @param array $form
   * @param \Drupal\Core\Form\FormStateInterface $form_state
   * @throws \Drupal\Component\Plugin\Exception\PluginException
   */
  public function generateSitemap(array &$form, FormStateInterface $form_state): void {
    $this->generator->generateSitemap();
  }

  /**
   * @param array $form
   * @param \Drupal\Core\Form\FormStateInterface $form_state
   * @throws \Drupal\Component\Plugin\Exception\PluginException
   */
  public function rebuildQueue(array &$form, FormStateInterface $form_state): void {
    $this->generator->rebuildQueue();
  }

}
